/*
 * decoder -- a program to decode the new pointer protocol
 *
 * Copyright (C) 1996 Alessandro Rubini (rubini@linux.it)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ********/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

#include "kmouse.h"
 
int main(int argc, char **argv)
{
    int kmfd, i,j;

    if (argc != 2) {
	fprintf(stderr,"%s: usage is \"%s /dev/kpointer\" or other device\n",
		argv[0], argv[0]);
	exit (1);
    }

    kmfd = open(argv[1],O_RDONLY);
    if (kmfd < 0) {
	fprintf(stderr,"%s: %s: %s\n",argv[0],argv[1],strerror(errno));
	exit(1);
    }

    while(1) {
	Kmouse_Packet packet;
	char *ptr;
	__s32 xtradim;

	/*
	 * This reader is pedantic: one should read the whole packet,
	 * and add then the missing dimemsions. A misaligned packet
	 * is definitely an error; go four bytes at a time anyway to show
	 * how to use the magic in the protocol. 4 bytes is the data atom
	 * of the protocol, so it wouldn't work to read less.
	 */

	do {
	    ptr = (char *)&packet;
	    i=read(kmfd,ptr,4);
	    if (i!=4) {
		printf("Closing device, exiting\n");
		close(kmfd); exit(0);
	    }
	    if (kmouse_packet_p(ptr))
                break;
            printf("Hmm... not a packet %04x\n",*(__u16 *)ptr);
        } while(1); /* if it exits, we have a packet */
        ptr += 4;
        i=read(kmfd,ptr,sizeof(packet)-4);
        if (i != sizeof(packet)-4) {
            printf("Hmm... short read for rest of packet (%i of %i)\n",
                   i, sizeof(packet)-4);
        }
        printf("%s, %iD %iB ",
               kmouse_absolute_p(&packet) ? "abs" : "rel",
               kmouse_nr_dimensions(&packet),
               kmouse_nr_buttons(&packet));
        printf("butt=%04x (%c%c%c%c) ",
               packet.buttons,
               packet.buttons & GPM_B_LEFT   ? 'L' : ' ',
               packet.buttons & GPM_B_MIDDLE ? 'M' : ' ',
               packet.buttons & GPM_B_RIGHT  ? 'R' : ' ',
               packet.buttons & ~(GPM_B_LEFT|GPM_B_MIDDLE|GPM_B_RIGHT)
               ? '+' : ' ');
        printf("% 8iX %8iY",packet.x,packet.y);
        /*
         * print extra dimensions if any
         */
        for (i=kmouse_nr_dimensions(&packet)-2, j=1; i; i--, j++) {
            if (!read(kmfd,&xtradim,4)) continue;
            printf(" % 8iZ%c",xtradim,j);
        }
        printf("\n");
    }
    exit(0);
}
